from qgis.core import QgsProcessing
from qgis.core import QgsProcessingAlgorithm
from qgis.core import QgsProcessingMultiStepFeedback
from qgis.core import QgsProcessingParameterNumber
from qgis.core import QgsProcessingParameterVectorLayer
from qgis.core import QgsProcessingParameterRasterLayer
from qgis.core import QgsProcessingParameterRasterDestination
import processing


class BerechnungAktErosionsrisiko(QgsProcessingAlgorithm):

    def initAlgorithm(self, config=None):
        self.addParameter(QgsProcessingParameterNumber('cfak', 'CP-Faktor', type=QgsProcessingParameterNumber.Double, minValue=1e-05, maxValue=1, defaultValue=0.1))
        self.addParameter(QgsProcessingParameterVectorLayer('feldblockkarte', 'Feldblockkarte', types=[QgsProcessing.TypeVectorAnyGeometry], defaultValue=None))
        self.addParameter(QgsProcessingParameterNumber('feldblocknumereingeben', 'Feldblocknummer eingeben', type=QgsProcessingParameterNumber.Integer, defaultValue=67496))
        self.addParameter(QgsProcessingParameterRasterLayer('potrisk', 'Potentielles Erosionsrisiko', defaultValue=None))
        self.addParameter(QgsProcessingParameterRasterDestination('Akt_erosionsrisiko', 'Akt_Erosionsrisiko', createByDefault=True, defaultValue=None))

    def processAlgorithm(self, parameters, context, model_feedback):
        # Use a multi-step feedback, so that individual child algorithm progress reports are adjusted for the
        # overall progress through the model
        feedback = QgsProcessingMultiStepFeedback(4, model_feedback)
        results = {}
        outputs = {}

        # Nach Attribut extrahieren
        alg_params = {
            'FIELD': 'Id',
            'INPUT': parameters['feldblockkarte'],
            'OPERATOR': 0,
            'VALUE': parameters['feldblocknumereingeben'],
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        outputs['NachAttributExtrahieren'] = processing.run('native:extractbyattribute', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        feedback.setCurrentStep(1)
        if feedback.isCanceled():
            return {}

        # Raster auf Layermaske zuschneiden
        alg_params = {
            'ALPHA_BAND': False,
            'CROP_TO_CUTLINE': True,
            'DATA_TYPE': 0,
            'INPUT': parameters['potrisk'],
            'KEEP_RESOLUTION': False,
            'MASK': outputs['NachAttributExtrahieren']['OUTPUT'],
            'MULTITHREADING': False,
            'NODATA': None,
            'OPTIONS': '',
            'SET_RESOLUTION': False,
            'SOURCE_CRS': None,
            'TARGET_CRS': None,
            'X_RESOLUTION': None,
            'Y_RESOLUTION': None,
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        outputs['RasterAufLayermaskeZuschneiden'] = processing.run('gdal:cliprasterbymasklayer', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        feedback.setCurrentStep(2)
        if feedback.isCanceled():
            return {}

        # Konstanten Rasterlayer erzeugen
        alg_params = {
            'EXTENT': outputs['RasterAufLayermaskeZuschneiden']['OUTPUT'],
            'NUMBER': parameters['cfak'],
            'PIXEL_SIZE': 0.1,
            'TARGET_CRS': 'ProjectCrs',
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        outputs['KonstantenRasterlayerErzeugen'] = processing.run('qgis:createconstantrasterlayer', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        feedback.setCurrentStep(3)
        if feedback.isCanceled():
            return {}

        # r.mapcalc.simple
        alg_params = {
            'GRASS_RASTER_FORMAT_META': '',
            'GRASS_RASTER_FORMAT_OPT': '',
            'GRASS_REGION_CELLSIZE_PARAMETER': 0,
            'GRASS_REGION_PARAMETER': None,
            'a': outputs['RasterAufLayermaskeZuschneiden']['OUTPUT'],
            'b': outputs['KonstantenRasterlayerErzeugen']['OUTPUT'],
            'c': None,
            'd': None,
            'e': None,
            'expression': 'A*B',
            'f': None,
            'output': parameters['Akt_erosionsrisiko']
        }
        outputs['Rmapcalcsimple'] = processing.run('grass7:r.mapcalc.simple', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        results['Akt_erosionsrisiko'] = outputs['Rmapcalcsimple']['output']
        return results

    def name(self):
        return 'berechnung akt. Erosionsrisiko'

    def displayName(self):
        return 'berechnung akt. Erosionsrisiko'

    def group(self):
        return ''

    def groupId(self):
        return ''

    def createInstance(self):
        return BerechnungAktErosionsrisiko()
